@@ -23,7 +23,6 @@ static /*@null@*/GWEN_PLUGIN_MANAGER *ab_pluginManagerImExporter=NULL;
 static /*@null@*/AB_BANKINFO_PLUGIN_LIST *ab_bankInfoPlugins=NULL;
 
 static /*@null@*/AB_IMEXPORTER_LIST *ab_imexporters=NULL;
-static /*@null@*/AB_PROVIDER_LIST *ab_providers=NULL;
 
 
 #define AB_DBIO_FOLDER "dbio"
@@ -61,9 +60,7 @@ int AB_Banking_PluginSystemInit(void)
     DBG_INFO(AQBANKING_LOGDOMAIN,
              "AqBanking v"
              AQBANKING_VERSION_FULL_STRING
-             " (compiled at "
-             COMPILE_DATETIME
-             "): initialising");
+             ": initialising");
 
     /* define locale paths */
     GWEN_PathManager_DefinePath(AB_PM_LIBNAME, AB_PM_LOCALEDIR);
@@ -303,7 +300,6 @@ int AB_Banking_PluginSystemInit(void)
 
 
     ab_imexporters=AB_ImExporter_List_new();
-    ab_providers=AB_Provider_List_new();
     ab_bankInfoPlugins=AB_BankInfoPlugin_List_new();
   }
   ab_plugin_init_count++;
@@ -318,8 +314,6 @@ int AB_Banking_PluginSystemFini(void)
     if (--ab_plugin_init_count==0) {
       AB_BankInfoPlugin_List_free(ab_bankInfoPlugins);
       ab_bankInfoPlugins=NULL;
-      AB_Provider_List_free(ab_providers);
-      ab_providers=NULL;
       AB_ImExporter_List_free(ab_imexporters);
       ab_imexporters=NULL;
 
