@@ -1,7 +1,7 @@
 /***************************************************************************
  begin       : Mon Mar 01 2004
  copyright   : (C) 2004 by Martin Preuss
- email       : martin@libchipcard.de
+ email       : martin at libchipcard.de
 
  ***************************************************************************
  *          Please see toplevel file COPYING for license details           *
@@ -12,17 +12,17 @@
 #define AQBANKING_VERSION_H
 
 #define AQBANKING_VERSION_MAJOR 6
-#define AQBANKING_VERSION_MINOR 2
-#define AQBANKING_VERSION_PATCHLEVEL 10
+#define AQBANKING_VERSION_MINOR 5
+#define AQBANKING_VERSION_PATCHLEVEL 3
 #define AQBANKING_VERSION_BUILD 0
 #define AQBANKING_VERSION_TAG "stable"
-#define AQBANKING_VERSION_FULL_STRING "6.2.10.0stable"
-#define AQBANKING_VERSION_STRING "6.2.10"
+#define AQBANKING_VERSION_FULL_STRING "6.5.3.0stable"
+#define AQBANKING_VERSION_STRING "6.5.3"
 
 
-#define AQBANKING_SO_CURRENT 46
-#define AQBANKING_SO_REVISION 10
-#define AQBANKING_SO_AGE 2
+#define AQBANKING_SO_CURRENT 49
+#define AQBANKING_SO_REVISION 3
+#define AQBANKING_SO_AGE 5
 #define AQBANKING_SO_EFFECTIVE 44
 
 #endif /* AQBANKING_VERSION_H */
