@@ -15,12 +15,12 @@
 
 #include "jobsepadebitsingle_p.h"
 #include "jobtransferbase_l.h"
-#include "aqhbci_l.h"
+#include "aqhbci/aqhbci_l.h"
 #include "accountjob_l.h"
-#include "job_l.h"
-#include "user_l.h"
-#include "provider_l.h"
-#include "hhd_l.h"
+#include "aqhbci/joblayer/job_l.h"
+#include "aqhbci/banking/user_l.h"
+#include "aqhbci/banking/provider_l.h"
+#include "aqhbci/applayer/hhd_l.h"
 
 #include <gwenhywfar/debug.h>
 #include <gwenhywfar/misc.h>
