@@ -14,12 +14,12 @@
 
 
 #include "jobsepastandingorderget_p.h"
-#include "aqhbci_l.h"
+#include "aqhbci/aqhbci_l.h"
 #include "accountjob_l.h"
-#include "job_l.h"
+#include "aqhbci/joblayer/job_l.h"
 #include "aqhbci/joblayer/job_swift.h"
 #include "aqhbci/joblayer/job_crypt.h"
-#include "user_l.h"
+#include "aqhbci/banking/user_l.h"
 
 #include <gwenhywfar/debug.h>
 #include <gwenhywfar/misc.h>
