@@ -15,6 +15,7 @@
 
 #include "cbox_queue.h"
 
+#include "aqhbci/admjobs/jobacknowledge_l.h"
 #include "aqhbci/admjobs/jobtan_l.h"
 
 #include "aqhbci/applayer/cbox_send.h"
@@ -33,6 +34,7 @@
  * ------------------------------------------------------------------------------------------------
  */
 
+static AH_JOBQUEUE *_createAckQueueFromTodoList(AB_USER *user, AH_JOB_LIST *jl, uint32_t jqFlags);
 static AH_JOBQUEUE *_createNextQueueFromTodoList(AB_USER *user, AH_JOB_LIST *jl, uint32_t jqFlags,
                                                  AH_JOB_LIST *finishedJobs);
 static int _performQueue(AH_OUTBOX_CBOX *cbox, AH_DIALOG *dlg, AH_JOBQUEUE *jq);
@@ -48,6 +50,8 @@ static void _handleQueueListError(AH_OUTBOX_CBOX *cbox, AH_JOBQUEUE_LIST *jql, c
 static int _sendAndRecvDialogQueues(AH_OUTBOX_CBOX *cbox);
 static int _sendAndRecvSelected(AH_OUTBOX_CBOX *cbox, uint32_t jqflags, uint32_t jqmask);
 static void _handleQueueError(AH_OUTBOX_CBOX *cbox, AH_JOBQUEUE *jq, const char *logStr);
+static AH_DIALOG *_pndqOpenDialog(AH_OUTBOX_CBOX *cbox, uint32_t jqflags);
+
 
 /* ------------------------------------------------------------------------------------------------
  * implementations
@@ -124,38 +128,139 @@ int _performQueue(AH_OUTBOX_CBOX *cbox, AH_DIALOG *dlg, AH_JOBQUEUE *jq)
   int rv;
   AB_USER *user;
   AH_JOB_LIST *finishedJobs;
+  int loopNum;
 
   user=AH_OutboxCBox_GetUser(cbox);
   finishedJobs=AH_OutboxCBox_GetFinishedJobs(cbox);
 
-  for (;;) {
+  for (loopNum=1; loopNum++;) {
+    AH_JOBQUEUE *jqAck;
     AH_JOBQUEUE *jqTodo;
     AH_JOB_LIST *jl;
 
+    DBG_INFO(AQHBCI_LOGDOMAIN, "Performing queue (loop %d)", loopNum);
+    if (GWEN_Logger_GetLevel(AQHBCI_LOGDOMAIN)>=GWEN_LoggerLevel_Info)
+      AH_JobQueue_DumpJobList(jq, stderr, 2);
+    else if (GWEN_Logger_GetLevel(AQHBCI_LOGDOMAIN)>=GWEN_LoggerLevel_Debug)
+      AH_JobQueue_Dump(jq, stderr, 2);
+
     jl=AH_JobQueue_TakeJobList(jq);
     assert(jl);
 
+    jqAck=_createAckQueueFromTodoList(user, jl, AH_JobQueue_GetFlags(jq));
     jqTodo=_createNextQueueFromTodoList(user, jl, AH_JobQueue_GetFlags(jq), finishedJobs);
     AH_Job_List_free(jl);
     AH_JobQueue_free(jq);
+
+    if (jqAck != NULL) {
+      DBG_INFO(AQHBCI_LOGDOMAIN, "Handling ACK jobs");
+      rv=AH_OutboxCBox_SendAndRecvQueue(cbox, dlg, jqAck);
+      if (rv) {
+        _handleQueueError(cbox, jqAck, "Error performing acknowledge queue");
+        return rv;
+      } /* if error during acknowledgement (jqAck freed by _handleQueueError */
+      AH_JobQueue_free(jqAck);
+    }
+
     if (jqTodo==NULL) {
       DBG_INFO(AQHBCI_LOGDOMAIN, "No more jobs left");
       break;
     }
-    jq=jqTodo;
-
-    /* jq now contains all jobs to be executed */
-    rv=AH_OutboxCBox_SendAndRecvQueue(cbox, dlg, jq);
-    if (rv) {
-      _handleQueueError(cbox, jq, "Error performing queue"); /* frees jobQueue */
-      return rv;
-    } /* if error */
+    else { 
+      DBG_INFO(AQHBCI_LOGDOMAIN, "Handling TODO jobs");
+      jq=jqTodo;
+      /* jq now contains all jobs to be executed */
+        
+      /* Execute NEXT send-recv round, synchrounously. */
+      rv=AH_OutboxCBox_SendAndRecvQueue(cbox, dlg, jq);
+      if (rv) {
+        _handleQueueError(cbox, jq, "Error performing queue"); /* frees jobQueue */
+        return rv;
+      } /* if error */
+    }
   } /* for */
 
   return 0;
 }
 
 
+AH_JOBQUEUE *_createAckQueueFromTodoList(AB_USER *user, AH_JOB_LIST *jl, uint32_t jqFlags)
+{
+  AH_JOB *j;
+  AH_JOBQUEUE *jqAck;
+
+  jqAck=AH_JobQueue_new(user);
+  /* copy some flags */
+  jqFlags&=~(AH_JOBQUEUE_FLAGS_CRYPT |
+             AH_JOBQUEUE_FLAGS_SIGN |
+             AH_JOBQUEUE_FLAGS_NOSYSID |
+             AH_JOBQUEUE_FLAGS_NOITAN);
+  AH_JobQueue_SetFlags(jqAck, (jqFlags&AH_JOBQUEUE_FLAGS_COPYMASK));
+
+  /* insert intermediate round for possible acknowledgements */
+  j=AH_Job_List_First(jl);
+  while (j) {
+    const char *jobName;
+
+    jobName=AH_Job_GetName(j);
+    if (!(jobName && *jobName))
+      jobName="<unnamed>";
+
+    if (AH_Job_GetStatus(j)==AH_JobStatusAnswered) {
+      /* Should we send an acknowledgement for the previously executed job? */
+      const void* ackCode = NULL;
+      AH_JOB* jAck = NULL;
+      unsigned int lenAckCode = 0;
+      GWEN_DB_NODE *args = AH_Job_GetArguments(j);
+
+      DBG_INFO(AQHBCI_LOGDOMAIN,
+               "Job \"%s\" with status \"answered\", checking whether it needs acknowledgement",
+               jobName);
+      ackCode = GWEN_DB_GetBinValue(args, "_tmpAckCode", 0, 0, 0, &lenAckCode);
+      if (ackCode != NULL && lenAckCode > 0) {
+         jAck = AH_Job_Acknowledge_new(AH_Job_GetProvider(j), AH_Job_GetUser(j), ackCode, lenAckCode);
+         DBG_NOTICE(AQHBCI_LOGDOMAIN, "Job \"%s\" received acknowledge code, prepare acknowledge job", jobName);
+         if (GWEN_DB_DeleteVar(args, "_tmpAckCode")) {
+           DBG_DEBUG(AQHBCI_LOGDOMAIN, "Temporary acknowledge code removed");
+         }
+      }
+      else {
+        DBG_DEBUG(AQHBCI_LOGDOMAIN, "Job \"%s\" didn't receive an acknowledge code, no acknowledge job needed.", jobName);
+      }
+
+      /* Job received acknowledge code in previous response and acknowledge is allowed by BPD and user wants to acknowledge - so do it. */
+      if (jAck != NULL) {
+        /* copy signers to new job */
+        if (AH_Job_GetFlags(j) & AH_JOB_FLAGS_SIGN) {
+          GWEN_STRINGLISTENTRY *se;
+          se=GWEN_StringList_FirstEntry(AH_Job_GetSigners(j));
+          while (se) {
+            AH_Job_AddSigner(jAck, GWEN_StringListEntry_Data(se));
+            se=GWEN_StringListEntry_Next(se);
+          } /* while */
+        }
+
+        if (AH_JobQueue_AddJob(jqAck, jAck)!=AH_JobQueueAddResultOk) {
+          DBG_DEBUG(AQHBCI_LOGDOMAIN, "Couldn't add ack job to todo list.");
+          AH_Job_SetStatus(j, AH_JobStatusError);
+        }
+        else {
+          AH_Job_Log(j, GWEN_LoggerLevel_Info, "Acknwoledge Job enqueued");
+        }
+      }
+    }
+    j=AH_Job_List_Next(j);
+  }
+
+  if (AH_JobQueue_GetCount(jqAck)==0) {
+    DBG_INFO(AQHBCI_LOGDOMAIN, "No acknwoledge jobs enqueued.");
+    AH_JobQueue_free(jqAck);
+    return NULL;
+  }
+
+  return jqAck;
+}
+
 
 AH_JOBQUEUE *_createNextQueueFromTodoList(AB_USER *user, AH_JOB_LIST *jl, uint32_t jqFlags, AH_JOB_LIST *finishedJobs)
 {
@@ -183,39 +288,44 @@ AH_JOBQUEUE *_createNextQueueFromTodoList(AB_USER *user, AH_JOB_LIST *jl, uint32
       DBG_INFO(AQHBCI_LOGDOMAIN,
 	       "Job \"%s\" with status \"answered\", checking whether it needs to be re-enqueued",
 	       jobName);
+      AB_Banking_LogMsgForJobId(AH_Job_GetBankingApi(j), AH_Job_GetId(j), "Job status \"answered\", checking for re-enqueing");
       /* prepare job for next message
        * (if attachpoint or multi-message job)
        */
       AH_Job_PrepareNextMessage(j);
       if (AH_Job_GetFlags(j) & AH_JOB_FLAGS_HASMOREMSGS) {
         DBG_NOTICE(AQHBCI_LOGDOMAIN, "Requeueing job \"%s\"", jobName);
+        AB_Banking_LogMsgForJobId(AH_Job_GetBankingApi(j), AH_Job_GetId(j), "Re-enqueueing job");
         /* we shall redo this job */
         if (AH_JobQueue_AddJob(jqTodo, j)!=AH_JobQueueAddResultOk) {
           DBG_ERROR(AQHBCI_LOGDOMAIN, "Job could not be re-added to queue, SNH!");
           AH_Job_Log(j, GWEN_LoggerLevel_Error, "Could not re-enqueue job");
+          AB_Banking_LogMsgForJobId(AH_Job_GetBankingApi(j), AH_Job_GetId(j), "Error re-enqueueing job");
           AH_Job_SetStatus(j, AH_JobStatusError);
         }
         else {
 	  AH_Job_Log(j, GWEN_LoggerLevel_Info, "Job re-enqueued (multi-message job)");
+          AB_Banking_LogMsgForJobId(AH_Job_GetBankingApi(j), AH_Job_GetId(j), "Job successfuly re-enqueued (multi-msg job)");
           j=NULL; /* mark that this job has been dealt with */
         }
       } /* if more messages */
       else {
-        DBG_NOTICE(AQHBCI_LOGDOMAIN, "Job \"%s\" has no messages left, not re-enqueing", jobName);
+        DBG_NOTICE(AQHBCI_LOGDOMAIN, "Job \"%s\" has no messages left, not re-enqueueing", jobName);
       }
     } /* if status "answered" */
 
     else if (AH_Job_GetStatus(j)==AH_JobStatusEnqueued) {
-      DBG_NOTICE(AQHBCI_LOGDOMAIN,
-		 "Job \"%s\" with status \"enqueued\", trying to re-enqueue (TODO: Is this really used?)",
-		 jobName);
+      DBG_NOTICE(AQHBCI_LOGDOMAIN, "Job \"%s\" with status \"enqueued\", trying to enqueue", jobName);
+      AB_Banking_LogMsgForJobId(AH_Job_GetBankingApi(j), AH_Job_GetId(j), "Job status \"enqueued\", trying to enqueue");
       if (AH_JobQueue_AddJob(jqTodo, j)!=AH_JobQueueAddResultOk) {
-        DBG_ERROR(AQHBCI_LOGDOMAIN, "Job could not be re-added to queue, SNH!");
+        DBG_ERROR(AQHBCI_LOGDOMAIN, "Job could not be added to queue, SNH!");
+        AB_Banking_LogMsgForJobId(AH_Job_GetBankingApi(j), AH_Job_GetId(j), "Error enqueueing job (internal)");
         AH_Job_SetStatus(j, AH_JobStatusError);
         AH_Job_Log(j, GWEN_LoggerLevel_Error, "Could not enqueue job");
       }
       else {
         AH_Job_Log(j, GWEN_LoggerLevel_Info, "Job enqueued (2)");
+        AB_Banking_LogMsgForJobId(AH_Job_GetBankingApi(j), AH_Job_GetId(j), "Job successfully enqueued (2)");
         j=NULL; /* mark that this job has been dealt with */
       }
     } /* if status "enqueued" */
@@ -225,6 +335,8 @@ AH_JOBQUEUE *_createNextQueueFromTodoList(AB_USER *user, AH_JOB_LIST *jl, uint32
 	       jobName,
 	       AH_Job_StatusName(AH_Job_GetStatus(j)),
                AH_Job_GetStatus(j));
+      AB_Banking_LogMsgForJobId(AH_Job_GetBankingApi(j), AH_Job_GetId(j), "Job status \"%s\", unexpected",
+                                AH_Job_StatusName(AH_Job_GetStatus(j)));
       if (GWEN_Logger_GetLevel(0)>=GWEN_LoggerLevel_Debug)
         AH_Job_Dump(j, stderr, 4);
     }
@@ -249,64 +361,23 @@ AH_JOBQUEUE *_createNextQueueFromTodoList(AB_USER *user, AH_JOB_LIST *jl, uint32
 
 int _performNonDialogQueues(AH_OUTBOX_CBOX *cbox, AH_JOBQUEUE_LIST *jql)
 {
-  AB_USER *user;
-  AB_PROVIDER *provider;
   AH_DIALOG *dlg;
   AH_JOBQUEUE *jq;
   int rv=0;
-  int i;
   uint32_t jqflags;
 
-  user=AH_OutboxCBox_GetUser(cbox);
-  provider=AH_OutboxCBox_GetProvider(cbox);
-
   if (AH_JobQueue_List_GetCount(jql)==0) {
     DBG_NOTICE(AQHBCI_LOGDOMAIN, "No queues to handle, doing nothing");
     AH_JobQueue_List_free(jql);
     return 0;
   }
 
-  for (i=0; i<2; i++) {
-    dlg=AH_Dialog_new(user, provider);
-    rv=AH_Dialog_Connect(dlg);
-    if (rv) {
-      DBG_INFO(AQHBCI_LOGDOMAIN,
-               "Could not begin a dialog for customer \"%s\" (%d)",
-               AB_User_GetCustomerId(user), rv);
-      /* finish all queues */
-      _handleQueueListError(cbox, jql, "Could not begin dialog");
-      AH_Dialog_free(dlg);
-      return rv;
-    }
-
-    jq=AH_JobQueue_List_First(jql);
-    jqflags=AH_JobQueue_GetFlags(jq);
-
-    /* open dialog */
-    rv=AH_OutboxCBox_OpenDialog(cbox, dlg, jqflags);
-    if (rv==0)
-      break;
-    else if (rv<0) {
-      DBG_INFO(AQHBCI_LOGDOMAIN, "Could not open dialog");
-      AH_Dialog_Disconnect(dlg);
-      /* finish all queues */
-      _handleQueueListError(cbox, jql, "Could not open dialog");
-      AH_Dialog_free(dlg);
-      return rv;
-    }
-    else if (rv==1) {
-      AH_Dialog_Disconnect(dlg);
-      AH_Dialog_free(dlg);
-      GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Info, I18N("Retrying to open dialog"));
-    }
-  }
-  if (rv) {
-    DBG_INFO(AQHBCI_LOGDOMAIN, "Could not open dialog");
-    AH_Dialog_Disconnect(dlg);
+  jqflags=AH_JobQueue_GetFlags(AH_JobQueue_List_First(jql));
+  dlg=_pndqOpenDialog(cbox, jqflags);
+  if (dlg==NULL) {
     /* finish all queues */
     _handleQueueListError(cbox, jql, "Could not open dialog");
-    AH_Dialog_free(dlg);
-    return rv;
+    return GWEN_ERROR_GENERIC;
   }
 
   /* handle queues */
@@ -344,6 +415,55 @@ int _performNonDialogQueues(AH_OUTBOX_CBOX *cbox, AH_JOBQUEUE_LIST *jql)
 
 
 
+AH_DIALOG *_pndqOpenDialog(AH_OUTBOX_CBOX *cbox, uint32_t jqflags)
+{
+  AB_PROVIDER *provider;
+  AB_USER *user;
+  int i;
+
+  user=AH_OutboxCBox_GetUser(cbox);
+  provider=AH_OutboxCBox_GetProvider(cbox);
+
+  for (i=0; i<2; i++) {
+    AH_DIALOG *dlg;
+    int rv;
+
+    dlg=AH_Dialog_new(user, provider);
+    rv=AH_Dialog_Connect(dlg);
+    if (rv) {
+      DBG_INFO(AQHBCI_LOGDOMAIN,
+               "Could not begin a dialog for customer \"%s\" (%d)",
+               AB_User_GetCustomerId(user), rv);
+      AH_Dialog_free(dlg);
+      return NULL;
+    }
+
+    /* open dialog */
+    rv=AH_OutboxCBox_OpenDialog(cbox, dlg, jqflags);
+    if (rv==0) {
+      DBG_INFO(AQHBCI_LOGDOMAIN, "Dialog open.");
+      return dlg;
+    }
+    else if (rv<0) {
+      DBG_INFO(AQHBCI_LOGDOMAIN, "Could not open dialog (%d)", rv);
+      AH_Dialog_Disconnect(dlg);
+      AH_Dialog_free(dlg);
+      return NULL;
+    }
+    else if (rv==1) {
+      /* TODO: Is this really needed?? I believe this was needed before when we had to try different SSL modi... */
+      DBG_INFO(AQHBCI_LOGDOMAIN, "Retrying to open dialog.");
+      AH_Dialog_Disconnect(dlg);
+      AH_Dialog_free(dlg);
+      GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Info, I18N("Retrying to open dialog"));
+    }
+  }
+  DBG_INFO(AQHBCI_LOGDOMAIN, "Could not open dialog");
+  return NULL;
+}
+
+
+
 int _performDialogQueue(AH_OUTBOX_CBOX *cbox, AH_JOBQUEUE *jq)
 {
   AB_USER *user;
@@ -626,10 +746,12 @@ void _handleQueueError(AH_OUTBOX_CBOX *cbox, AH_JOBQUEUE *jq, const char *logStr
   while ((j=AH_Job_List_First(jl))) {
     AH_Job_List_Del(j);
     if (AH_Job_GetStatus(j)!=AH_JobStatusAnswered) {
+      if (logStr) {
+        AH_Job_Log(j, GWEN_LoggerLevel_Error, logStr);
+        AB_Banking_LogMsgForJobId(AH_Job_GetBankingApi(j), AH_Job_GetId(j), logStr);
+      }
       DBG_INFO(AQHBCI_LOGDOMAIN, "Setting status of job \"%s\" to ERROR", AH_Job_GetName(j));
       AH_Job_SetStatus(j, AH_JobStatusError);
-      if (logStr)
-        AH_Job_Log(j, GWEN_LoggerLevel_Error, logStr);
     }
     AH_Job_List_Add(j, finishedJobs);
   }
