@@ -1,6 +1,6 @@
 /***************************************************************************
     begin       : Mon Mar 01 2004
-    copyright   : (C) 2018 by Martin Preuss
+    copyright   : (C) 2022 by Martin Preuss
     email       : martin@libchipcard.de
 
  ***************************************************************************
@@ -13,9 +13,6 @@
 
 #include "job_commit.h"
 #include "job_commit_key.h"
-#include "job_commit_bpd.h"
-#include "job_commit_account.h"
-#include "job_commit_key.h"
 #include "aqhbci/banking/user_l.h"
 #include "aqhbci/banking/account_l.h"
 
@@ -134,31 +131,12 @@ void AH_Job_ReadAccountDataSeg(AB_ACCOUNT *acc, GWEN_DB_NODE *dbAccountData)
 
 int _commitSystemData(AH_JOB *j, int doLock)
 {
-  int rv;
-
   DBG_NOTICE(AQHBCI_LOGDOMAIN, "Committing data");
   /* GWEN_DB_Dump(j->jobResponses, 2); */
 
   DBG_DEBUG(AQHBCI_LOGDOMAIN, "Reading segment results, bank messages etc");
   _readSomeKnownSegments(j, AH_Job_GetResponses(j));
 
-  /* try to extract bank parameter data */
-  DBG_DEBUG(AQHBCI_LOGDOMAIN, "Committing BPD");
-  rv=AH_Job_Commit_Bpd(j);
-  if (rv<0) {
-    DBG_INFO(AQHBCI_LOGDOMAIN, "here (%d)", rv);
-    return rv;
-  }
-
-  /* try to extract accounts */
-  if (AH_Job_GetFlags(j) & AH_JOB_FLAGS_IGNOREACCOUNTS) {
-    DBG_INFO(AQHBCI_LOGDOMAIN, "Ignoring possibly received accounts");
-  }
-  else {
-    DBG_INFO(AQHBCI_LOGDOMAIN, "Committing accounts");
-    AH_Job_Commit_Accounts(j);
-  }
-
   DBG_NOTICE(AQHBCI_LOGDOMAIN, "Finished.");
   return 0;
 }
